@@ -32,7 +32,9 @@
  */
 #include "plr.h"
 
+#include "miscadmin.h"
 #include "optimizer/clauses.h"
+#include "utils/memutils.h"
 
 /* Example format: "/usr/local/pgsql/lib" */
 #ifndef PKGLIBDIR
@@ -1703,7 +1705,7 @@ compute_function_hashkey(FunctionCallInfo fcinfo,
 	/* get the argument types */
 	for (i = 0; i < procStruct->pronargs; i++)
 	{
-		Oid			argtypeid = procStruct->proargtypes[i];
+		Oid			argtypeid = PROARGTYPES(i);
 
 		/*
 		 * Check for polymorphic arguments. If found, use the actual
@@ -1843,6 +1845,8 @@ get_lib_pathstr(Oid funcid)
 	tmp = SysCacheGetAttr(PROCOID, procedureTuple, Anum_pg_proc_probin, &isnull);
 	raw_path = DatumGetCString(DirectFunctionCall1(byteaout, tmp));
 	cooked_path = expand_dynamic_library_name(raw_path);
+	if (!cooked_path)
+		cooked_path = pstrdup(raw_path);
 
 	ReleaseSysCache(procedureTuple);
 
@@ -1853,8 +1857,14 @@ char *
 get_load_self_ref_cmd(Oid funcid)
 {
 	char   *libstr = get_lib_pathstr(funcid);
-	char   *buf = (char *) palloc(strlen(libstr) + 12 + 1);
+	char   *buf = NULL;
 
+	if (libstr)
+		buf = (char *) palloc(strlen(libstr) + 12 + 1);
+	else
+		ereport(ERROR,
+			(errcode_for_file_access(),
+			errmsg("could not find path to PL/R shared library")));
 	sprintf(buf, "dyn.load(\"%s\")", libstr);
 	return buf;
 }
@@ -1883,6 +1893,10 @@ file_exists(const char *name)
 	return false;
 }
 
+#ifndef DLSUFFIX
+#error "DLSUFFIX must be defined to compile this file."
+#endif
+
 /*
  * If name contains a slash, check if the file exists, if so return
  * the name.  Else (no slash) try to expand using search path (see
@@ -1941,6 +1955,7 @@ expand_dynamic_library_name(const char *name)
 }
 
 
+#if defined(PG_VERSION_73_COMPAT) || defined(PG_VERSION_74_COMPAT)
 /*
  * Substitute for any macros appearing in the given string.
  * Result is always freshly palloc'd.
@@ -1980,6 +1995,41 @@ substitute_libpath_macro(const char *name)
 	}
 }
 
+#else
+
+/*
+ * Substitute for any macros appearing in the given string.
+ * Result is always freshly palloc'd.
+ */
+static char *
+substitute_libpath_macro(const char *name)
+{
+	const char *sep_ptr;
+	char	   *ret;
+
+	AssertArg(name != NULL);
+
+	if (name[0] != '$')
+		return pstrdup(name);
+
+	if ((sep_ptr = first_dir_separator(name)) == NULL)
+		sep_ptr = name + strlen(name);
+
+	if (strlen("$libdir") != sep_ptr - name ||
+		strncmp(name, "$libdir", strlen("$libdir")) != 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_NAME),
+				errmsg("invalid macro name in dynamic library path: %s", name)));
+
+	ret = palloc(strlen(pkglib_path) + strlen(sep_ptr) + 1);
+
+	strcpy(ret, pkglib_path);
+	strcat(ret, sep_ptr);
+
+	return ret;
+}
+
+#endif /* PG_VERSION_73_COMPAT || PG_VERSION_74_COMPAT */
 
 /*
  * Search for a file called 'basename' in the colon-separated search
